﻿Option Strict On


Imports System.Net
Imports System.IO
Imports System.Collections.Specialized
Imports System.Threading

Namespace Sezz.AION

    Public Class Armory

        Private cAionArmoryCache As New StringDictionary
        Private sAionArmoryCacheFile As String = Application.StartupPath & "\" & Application.ProductName & ".AionArmoryCache"

        ' Multi-Threading
        Private qDownloadManager As Queue
        Private oAionArmoryUpdateThread As Thread
        Public Event DownloadFinished(ByVal oData As Sezz.AION.Armory.Data)

        Public Sub LoadCache()
            If File.Exists(sAionArmoryCacheFile) Then
                Try
                    Dim fs As New FileStream(sAionArmoryCacheFile, FileMode.Open)
                    Dim bf As New Runtime.Serialization.Formatters.Binary.BinaryFormatter()
                    cAionArmoryCache = CType(bf.Deserialize(fs), StringDictionary)
                    fs.Close()
                Catch ex As Exception
                    Debug.WriteLine(ex.Message)
                End Try
            End If
        End Sub

        Public Sub UpdateCache()
            Try
                If File.Exists(sAionArmoryCacheFile) Then File.Delete(sAionArmoryCacheFile)
                Dim fs As New FileStream(sAionArmoryCacheFile, FileMode.CreateNew)
                Dim bf As New Runtime.Serialization.Formatters.Binary.BinaryFormatter()
                bf.Serialize(fs, cAionArmoryCache)
                fs.Close()
            Catch ex As Exception
                Debug.WriteLine(ex.Message)
            End Try
        End Sub

        Private Function GetRegularExpressionMatches(ByVal sString As String, ByVal sRegularExpression As String) As String()
            Dim oRegExp As New System.Text.RegularExpressions.Regex(sRegularExpression)
            Dim sMatches() As String = {}
            Dim oRegExpMatch As System.Text.RegularExpressions.Group
            Dim iCurrent As Integer = 0

            If oRegExp.IsMatch(sString) Then
                ReDim sMatches(oRegExp.Match(sString).Groups.Count - 1)
                For Each oRegExpMatch In oRegExp.Match(sString).Groups
                    sMatches(iCurrent) = oRegExpMatch.Value
                    iCurrent += 1
                Next
            End If

            Return sMatches
        End Function

        Private Function GetRegularExpressionMatchString(ByVal sString As String, ByVal sRegularExpression As String) As String
            Dim sMatches() As String = GetRegularExpressionMatches(sString, sRegularExpression)
            If sMatches.Length = 2 Then Return sMatches(1) Else Return ""
        End Function

        Private Function GetRegularExpressionMatchInteger(ByVal sString As String, ByVal sRegularExpression As String) As Integer
            Dim sMatches() As String = GetRegularExpressionMatches(sString, sRegularExpression)
            If sMatches.Length = 2 Then Return CType(sMatches(1), Integer) Else Return 0
        End Function

        Public Enum RequestType
            Item = 1
            ' (NYI) Quest = 4
            ' (NYI) NPC = ?
            Spell = 6
            ' (NYI) Recipe = 131
        End Enum

        Public Structure Data
            Dim Name As String
            Dim ID As Integer
            Dim Rarity As Byte
            Dim Icon As Integer
            Dim Type As Sezz.AION.Armory.RequestType
        End Structure

        Private Function ParseData(ByVal sData As String) As Sezz.AION.Armory.Data
            ParseData.Name = GetRegularExpressionMatchString(sData, "name\\"">([^<]*)</span")
            ParseData.ID = GetRegularExpressionMatchInteger(sData, "{id:([^,]*),")
            ParseData.Rarity = CType(GetRegularExpressionMatchInteger(sData, ",rarity:([0-9]*)"), Byte)
            ParseData.Icon = GetRegularExpressionMatchInteger(sData, ",icon:([0-9]*)")
        End Function

        Public Function GetData(ByVal lId As Integer, ByVal iType As Sezz.AION.Armory.RequestType, Optional ByVal bDownloadInBackground As Boolean = False, Optional ByVal bOverrideCache As Boolean = False) As Sezz.AION.Armory.Data
            Dim oData As Sezz.AION.Armory.Data

            If Not bOverrideCache AndAlso cAionArmoryCache.ContainsKey(iType & "|" & lId) Then
                oData = ParseData(cAionArmoryCache(iType & "|" & lId))
                If bDownloadInBackground Then
                    RaiseEvent DownloadFinished(oData)
                End If
            Else
                If bDownloadInBackground Then
                    ' Use Download Manager Thread
                    QueueDownload(lId, iType, bOverrideCache)
                    oData = New Sezz.AION.Armory.Data
                    With oData
                        .Name = ""
                        .ID = lId
                        .Type = iType
                    End With

                    ' Start Download Manager
                    If oAionArmoryUpdateThread Is Nothing OrElse oAionArmoryUpdateThread.ThreadState = ThreadState.Stopped Then oAionArmoryUpdateThread = New Thread(AddressOf DownloadManager)
                    If Not oAionArmoryUpdateThread.IsAlive Then oAionArmoryUpdateThread.Start()
                Else
                    If Not bOverrideCache AndAlso cAionArmoryCache.ContainsKey(iType & "|" & lId) Then
                        Debug.WriteLine("[Sezz.AION.Armory] InCache: " & lId)
                        oData = ParseData(cAionArmoryCache(iType & "|" & lId))
                    Else
                        Dim sResponse As String = DownloadURL("http://www.aionarmory.com/ExTooltips.aspx?id=" & lId & "&type=" & iType)

                        If Not cAionArmoryCache.ContainsKey(iType & "|" & lId) Then cAionArmoryCache.Add(iType & "|" & lId, sResponse) _
                        Else cAionArmoryCache(iType & "|" & lId) = sResponse

                        oData = ParseData(sResponse)
                    End If
                End If
            End If

            Return oData
        End Function

#Region " Download "
        Private Sub QueueDownload(ByVal lId As Integer, ByVal iType As Sezz.AION.Armory.RequestType, ByVal bOverrideCache As Boolean)
            Dim Params As Object()
            For Each Params In qDownloadManager
                If CType(Params(0), Integer) = lId Then
                    Exit Sub
                End If
            Next
            qDownloadManager.Enqueue(New Object() {lId, iType, bOverrideCache})
        End Sub

        Public Sub DownloadManager()
            While (qDownloadManager.Count > 0)
                Debug.WriteLine("[Sezz.AION.Armory] [Download] Queue: " & qDownloadManager.Count)
                Download()
            End While
        End Sub

        Private Sub Download()
            ' Downloads the first item from the queue and updates the cache
            Dim Params As Object()
            If qDownloadManager.Count > 0 Then
                Params = CType(qDownloadManager.Dequeue, Object())

                Dim lId As Integer = CType(Params(0), Integer)
                Dim iType As Sezz.AION.Armory.RequestType = CType(Params(1), Sezz.AION.Armory.RequestType)
                Dim bOverrideCache As Boolean = CType(Params(2), Boolean)

                Debug.Write("[Sezz.AION.Armory] [Download] " & lId & "/" & iType)

                If Not bOverrideCache AndAlso cAionArmoryCache.ContainsKey(iType & "|" & lId) Then
                    Debug.WriteLine(" [Cached]")
                    RaiseEvent DownloadFinished(ParseData(cAionArmoryCache(iType & "|" & lId)))
                Else
                    Dim sResponse As String = DownloadURL("http://www.aionarmory.com/ExTooltips.aspx?id=" & lId & "&type=" & iType)
                    If sResponse.Length > 0 Then
                        If Not cAionArmoryCache.ContainsKey(iType & "|" & lId) Then cAionArmoryCache.Add(iType & "|" & lId, sResponse) _
                        Else cAionArmoryCache(iType & "|" & lId) = sResponse
                    End If

                    Debug.WriteLine(" [Done]")
                    RaiseEvent DownloadFinished(ParseData(sResponse))
                End If
            Else
                RaiseEvent DownloadFinished(New Sezz.AION.Armory.Data)
            End If
        End Sub

        Private Function DownloadURL(ByVal sURL As String) As String
            Dim sResponse As String = ""
            Dim hwrRequest As HttpWebRequest = CType(System.Net.WebRequest.Create(New Uri(sURL)), HttpWebRequest)

            hwrRequest.Accept = "*/*"
            hwrRequest.UserAgent = "Mozilla/5.0 (Windows; U; Windows NT 6.1; de; rv:1.9.1.3) Gecko/20090824 Firefox/3.5.3 (.NET CLR 3.5.30729)"
            hwrRequest.Headers.Add(HttpRequestHeader.AcceptLanguage, "en-us")
            hwrRequest.Headers.Add(HttpRequestHeader.Pragma, "no-cache")
            hwrRequest.Timeout = 10000
            Try
                Using wrResponse As WebResponse = hwrRequest.GetResponse()
                    Using srReader As New StreamReader(wrResponse.GetResponseStream(), System.Text.Encoding.Unicode)
                        sResponse = srReader.ReadToEnd()
                    End Using
                End Using
            Catch wex As WebException
                If wex.Response Is Nothing Then Throw wex
                If CType(wex.Response, HttpWebResponse).StatusCode = HttpStatusCode.NotFound Then
                    sResponse = ""
                Else
                    Throw wex
                End If
            End Try
            hwrRequest = Nothing

            Return sResponse
        End Function
#End Region

        Public Sub New()
            qDownloadManager = New Queue
        End Sub
    End Class

End Namespace