﻿Imports System.Runtime.InteropServices

Namespace Sezz.AION

    Public Class Memory

        <Flags()> Private Enum ProcessAccessFlags As Integer
            ''' <summary>Specifies all possible access flags for the process object.</summary>
            AllAccess = CreateThread Or DuplicateHandle Or QueryInformation Or SetInformation Or Terminate Or VMOperation Or VMRead Or VMWrite Or Synchronize
            ''' <summary>Enables usage of the process handle in the CreateRemoteThread function to create a thread in the process.</summary>
            CreateThread = &H2
            ''' <summary>Enables usage of the process handle as either the source or target process in the DuplicateHandle function to duplicate a handle.</summary>
            DuplicateHandle = &H40
            ''' <summary>Enables usage of the process handle in the GetExitCodeProcess and GetPriorityClass functions to read information from the process object.</summary>
            QueryInformation = &H400
            ''' <summary>Enables usage of the process handle in the SetPriorityClass function to set the priority class of the process.</summary>
            SetInformation = &H200
            ''' <summary>Enables usage of the process handle in the TerminateProcess function to terminate the process.</summary>
            Terminate = &H1
            ''' <summary>Enables usage of the process handle in the VirtualProtectEx and WriteProcessMemory functions to modify the virtual memory of the process.</summary>
            VMOperation = &H8
            ''' <summary>Enables usage of the process handle in the ReadProcessMemory function to' read from the virtual memory of the process.</summary>
            VMRead = &H10
            ''' <summary>Enables usage of the process handle in the WriteProcessMemory function to write to the virtual memory of the process.</summary>
            VMWrite = &H20
            ''' <summary>Enables usage of the process handle in any of the wait functions to wait for the process to terminate.</summary>
            Synchronize = &H100000
        End Enum

        <DllImport("kernel32.dll")> _
        Private Shared Function OpenProcess(ByVal dwDesiredAccess As ProcessAccessFlags, <MarshalAs(UnmanagedType.Bool)> ByVal bInheritHandle As Boolean, ByVal dwProcessId As Integer) As IntPtr
        End Function

        <DllImport("kernel32.dll", SetLastError:=True)> _
        Private Shared Function CloseHandle(ByVal hHandle As IntPtr) As Boolean
        End Function

        <DllImport("kernel32.dll", SetLastError:=True)> _
        Public Shared Function ReadProcessMemory(ByVal hProcess As IntPtr, ByVal lpBaseAddress As IntPtr, <Out()> ByVal lpBuffer() As Byte, ByVal iSize As Integer, ByRef lpNumberOfBytesRead As Integer) As Boolean
        End Function

        Public Shared Function GetModuleBaseAddress(ByVal iProcessID As Integer, ByVal sName As String) As IntPtr
            If iProcessID > 0 Then
                Dim myProcess As System.Diagnostics.Process = System.Diagnostics.Process.GetProcessById(iProcessID)
                Dim myProcessModule As System.Diagnostics.ProcessModule

                For Each myProcessModule In myProcess.Modules
                    If myProcessModule.ModuleName = sName Then
                        Debug.WriteLine("ModuleName: " & myProcessModule.ModuleName)
                        Debug.WriteLine("FileName: " & myProcessModule.FileName)
                        Debug.WriteLine("BaseAddress: " & myProcessModule.BaseAddress.ToString) ' :)
                        Debug.WriteLine("EntryPointAddress: " & myProcessModule.EntryPointAddress.ToString)

                        Return myProcessModule.BaseAddress
                    End If
                Next

                Return IntPtr.Zero
            End If
        End Function

        Public Shared Function ReadStringValue(ByVal iProcessID As Integer, ByVal iAddress As Long, ByVal iLength As Integer, Optional ByVal bUnicode As Boolean = False) As String
            Dim btBuffer(iLength) As Byte
            Dim sRetValue As String = ""
            Dim iProcessHandle As IntPtr
            Dim iBytesRead As Integer
            Dim bReadMemory As Boolean

            iProcessHandle = OpenProcess(ProcessAccessFlags.CreateThread Or ProcessAccessFlags.QueryInformation Or ProcessAccessFlags.VMOperation Or ProcessAccessFlags.VMRead, False, iProcessID)
            If iProcessHandle <> IntPtr.Zero Then
                bReadMemory = ReadProcessMemory(iProcessHandle, iAddress, btBuffer, btBuffer.Length, iBytesRead)
                CloseHandle(iProcessHandle)
            End If

            If bReadMemory Then
                If bUnicode Then sRetValue = System.Text.Encoding.Unicode.GetString(btBuffer) Else sRetValue = System.Text.Encoding.UTF8.GetString(btBuffer)
                If sRetValue.IndexOf(Chr(0)) <> -1 Then sRetValue = sRetValue.Remove(sRetValue.IndexOf(Chr(0)))
            End If

            Return sRetValue
        End Function

        Public Shared Function ReadIntegerValue(ByVal iProcessID As Integer, ByVal iAddress As Long) As Integer
            Dim btBuffer(Marshal.SizeOf(New Integer)) As Byte
            Dim iProcessHandle As IntPtr
            Dim bReadMemory As Boolean
            Dim iBytesRead As Integer

            iProcessHandle = OpenProcess(ProcessAccessFlags.CreateThread Or ProcessAccessFlags.QueryInformation Or ProcessAccessFlags.VMOperation Or ProcessAccessFlags.VMRead, False, iProcessID)
            If iProcessHandle <> IntPtr.Zero Then
                bReadMemory = ReadProcessMemory(iProcessHandle, iAddress, btBuffer, Marshal.SizeOf(New Integer), iBytesRead)
                CloseHandle(iProcessHandle)
            End If

            If bReadMemory Then Return BitConverter.ToInt32(btBuffer, 0) Else Return Integer.MaxValue
        End Function

        Public Shared Function ReadFloatValue(ByVal iProcessID As Integer, ByVal iAddress As Long) As Single
            Dim btBuffer(Marshal.SizeOf(New Single)) As Byte
            Dim iProcessHandle As IntPtr
            Dim bReadMemory As Boolean
            Dim iBytesRead As Integer

            iProcessHandle = OpenProcess(ProcessAccessFlags.CreateThread Or ProcessAccessFlags.QueryInformation Or ProcessAccessFlags.VMOperation Or ProcessAccessFlags.VMRead, False, iProcessID)
            If iProcessHandle <> IntPtr.Zero Then
                bReadMemory = ReadProcessMemory(iProcessHandle, iAddress, btBuffer, Marshal.SizeOf(New Single), iBytesRead)
                CloseHandle(iProcessHandle)
            End If

            If bReadMemory Then Return BitConverter.ToSingle(btBuffer, 0) Else Return Single.MaxValue
        End Function

        Public Shared Function ReadByteValue(ByVal iProcessID As Integer, ByVal iAddress As Long) As Byte
            Dim btBuffer(Marshal.SizeOf(New Byte)) As Byte
            Dim iProcessHandle As IntPtr
            Dim bReadMemory As Boolean
            Dim iBytesRead As Integer

            iProcessHandle = OpenProcess(ProcessAccessFlags.CreateThread Or ProcessAccessFlags.QueryInformation Or ProcessAccessFlags.VMOperation Or ProcessAccessFlags.VMRead, False, iProcessID)
            If iProcessHandle <> IntPtr.Zero Then
                bReadMemory = ReadProcessMemory(iProcessHandle, iAddress, btBuffer, Marshal.SizeOf(New Byte), iBytesRead)
                CloseHandle(iProcessHandle)
            End If

            If bReadMemory Then Return btBuffer(0) Else Return Byte.MaxValue
        End Function
    End Class

End Namespace