﻿Namespace Sezz.AION

    Public Class Player

        Private _Game As Sezz.AION.Game

        Private _sName As String
        Private _sLegion As String
        Private _iLevel As Byte
        Private _iCurrentHP As Integer
        Private _iMaxHP As Integer
        Private _iCurrentMP As Integer
        Private _iMaxMP As Integer
        Private _iExperience As Integer
        Private _iMaxExperience As Integer
        Private _bIsCasting As Boolean
        Private _bIsGathering As Boolean
        Private _iCastingSpellId As Integer
        Private _iGatheringItemId As Integer
        Private _iX As Single
        Private _iY As Single
        Private _iZ As Single
        Private _bIsLoggedIn As Boolean

        Public ReadOnly Property IsCasting() As Boolean
            Get
                Return _bIsCasting
            End Get
        End Property

        Public ReadOnly Property IsLoggedIn() As Boolean
            Get
                Return _bIsLoggedIn
            End Get
        End Property

        Public ReadOnly Property IsAlive() As Boolean
            Get
                Return _iCurrentHP > 0
            End Get
        End Property

        Public ReadOnly Property CastingSpellId() As Integer
            Get
                Return _iCastingSpellId
            End Get
        End Property

        Public ReadOnly Property GatheringItemId() As Integer
            Get
                Return _iGatheringItemId
            End Get
        End Property

        Public ReadOnly Property IsGathering() As Boolean
            Get
                Return _bIsGathering
            End Get
        End Property

        Public ReadOnly Property Name() As String
            Get
                Return _sName
            End Get
        End Property

        Public ReadOnly Property Legion() As String
            Get
                Return _sLegion
            End Get
        End Property

        Public ReadOnly Property HP() As Integer()
            Get
                Return New Integer() {_iCurrentHP, _iMaxHP}
            End Get
        End Property

        Public ReadOnly Property MP() As Integer()
            Get
                Return New Integer() {_iCurrentMP, _iMaxMP}
            End Get
        End Property

        Public ReadOnly Property X() As Single
            Get
                Return _iX
            End Get
        End Property

        Public ReadOnly Property Y() As Single
            Get
                Return _iY
            End Get
        End Property

        Public ReadOnly Property Z() As Single
            Get
                Return _iZ
            End Get
        End Property

        Public ReadOnly Property Level() As Byte
            Get
                Return _iLevel
            End Get
        End Property

        Public ReadOnly Property FullName() As String
            Get
                Dim sFullName As String = Name & " (" & Level.ToString & ")"
                If Legion.Length > 0 Then
                    sFullName = sFullName & " <" & Legion & ">"
                End If
                Return sFullName
            End Get
        End Property

        Public ReadOnly Property Experience() As Integer
            Get
                Return _iExperience
            End Get
        End Property

        Public ReadOnly Property MaxExperience() As Integer
            Get
                Return _iMaxExperience
            End Get
        End Property

        Public ReadOnly Property CompleteExperience() As Integer
            Get
                Dim iCompleteExperience As UInteger = 0

                If Level > 1 Then
                    For iCntLevel As Single = 1 To Level - 1
                        iCompleteExperience += _Game.ExperienceToLevel(iCntLevel - 1)
                    Next
                End If

                Return iCompleteExperience + Experience
            End Get
        End Property

        Public ReadOnly Property DistanceTo(ByVal Entity As Sezz.AION.Entity) As Double
            Get
                ' AionMemory.Tools Distance3D()
                '      __________________________________
                ' d = √ (x2-x1)^2 + (y2-y1)^2 + (z2-z1)^2
                '
                Dim Result As Double = 0
                Dim part1 As Double = System.Math.Pow((Entity.X() - Me.X()), 2)
                Dim part2 As Double = System.Math.Pow((Entity.Y() - Me.Y()), 2)
                Dim part3 As Double = System.Math.Pow((Entity.Z() - Me.Z()), 2)
                Dim underRadical As Double = part1 + part2 + part3
                Result = System.Math.Sqrt(underRadical)
                Return Result
            End Get
        End Property

        Public Function Update() As Boolean
            If _Game.Running Then
                _bIsLoggedIn = (Sezz.AION.Memory.ReadIntegerValue(_Game.ProcessID, _Game.GameDllBaseAddress.ToInt64 + &H8E4444) > 0)
                If _bIsLoggedIn Then
                    _sName = Sezz.AION.Memory.ReadStringValue(_Game.ProcessID, _Game.GameDllBaseAddress.ToInt64 + &H925F68, 32, True) ' 1.5.0.11
                    _sLegion = Sezz.AION.Memory.ReadStringValue(_Game.ProcessID, _Game.GameDllBaseAddress.ToInt64 + &H8E4DB0, 64, True) ' 1.5.0.11
                    _iCurrentHP = Sezz.AION.Memory.ReadIntegerValue(_Game.ProcessID, _Game.GameDllBaseAddress.ToInt64 + &H8EEEB8) ' 1.5.0.11
                    _iMaxHP = Sezz.AION.Memory.ReadIntegerValue(_Game.ProcessID, _Game.GameDllBaseAddress.ToInt64 + &H8EEEB4) ' 1.5.0.11
                    _iCurrentMP = Sezz.AION.Memory.ReadIntegerValue(_Game.ProcessID, _Game.GameDllBaseAddress.ToInt64 + &H8EEEC0) ' 1.5.0.11
                    _iMaxMP = Sezz.AION.Memory.ReadIntegerValue(_Game.ProcessID, _Game.GameDllBaseAddress.ToInt64 + &H8EEEBC) ' 1.5.0.11
                    _iLevel = Sezz.AION.Memory.ReadByteValue(_Game.ProcessID, _Game.GameDllBaseAddress.ToInt64 + &H8EEE90) ' 1.5.0.11
                    _iCastingSpellId = Sezz.AION.Memory.ReadIntegerValue(_Game.ProcessID, _Game.GameDllBaseAddress.ToInt64 + &H8E4CB8) ' 1.5.0.11
                    _bIsCasting = (_iCastingSpellId > 0)
                    _iGatheringItemId = Sezz.AION.Memory.ReadIntegerValue(_Game.ProcessID, _Game.GameDllBaseAddress.ToInt64 + &H8E655C)
                    _bIsGathering = (_iGatheringItemId > 0)
                    _iX = Sezz.AION.Memory.ReadFloatValue(_Game.ProcessID, (_Game.GameDllBaseAddress.ToInt64 + &H8E6940)) ' 1.5.0.11
                    _iY = Sezz.AION.Memory.ReadFloatValue(_Game.ProcessID, (_Game.GameDllBaseAddress.ToInt64 + &H8E6944)) ' 1.5.0.11
                    _iZ = Sezz.AION.Memory.ReadFloatValue(_Game.ProcessID, (_Game.GameDllBaseAddress.ToInt64 + &H8E6948)) ' 1.5.0.11
                    _iExperience = Sezz.AION.Memory.ReadIntegerValue(_Game.ProcessID, (_Game.GameDllBaseAddress.ToInt64 + &H8EEEA8)) ' 1.5.0.11
                    _iMaxExperience = Sezz.AION.Memory.ReadIntegerValue(_Game.ProcessID, (_Game.GameDllBaseAddress.ToInt64 + &H8EEE98)) ' 1.5.0.11
                    Return True
                Else
                    Return False
                End If
            Else
                Return False
            End If
        End Function

        Public Sub New(ByRef Game As Sezz.AION.Game)
            _Game = Game
            _sName = ""
            _sLegion = ""
            _iCurrentHP = 0
            _iMaxHP = 0
            _iCurrentMP = 0
            _iMaxMP = 0
            _iExperience = 0
            _iMaxExperience = 0
            _iLevel = 0
            _iGatheringItemId = 0
            _bIsCasting = False
            _iGatheringItemId = 0
            _bIsGathering = False
            _bIsLoggedIn = False
            _iX = 0
            _iY = 0
            _iZ = 0
        End Sub
    End Class

End Namespace