Option Explicit On 
Option Strict On

Imports System.Xml
Imports System.Runtime.InteropServices

' XML Settings Interface v1.0
' PIXEL.LI, Martin Karer, 2003

' ChangeLog
' 2003-05-30 18:16 Added getValue and setValue methods
' 2003-05-30 16:42 Release: v1.0
' 2003-05-30 16:26 Feature added: Only save CHANGED Settings
' 2003-05-28 22:46 File created

Namespace Sezz.XML

#Region " Class: Settings "
    ' Settings
    Public Class Settings
        Private m_htSettings As Hashtable
        Private m_xmlDocument As XmlDocument
        Private m_sFilename As String

#Region " Constructor "
        Sub New(Optional ByVal sFileName As String = "")
            ' Initialise
            If sFileName.Length = 0 Then sFileName = Application.StartupPath & "\" & Application.ProductName & ".Configuration.xml"
            m_htSettings = New Hashtable
            m_sFilename = sFileName

            ' Load XML Document
            initialiseXmlDocument()

            ' Fill Settings Collection
            loadSettings()
        End Sub
#End Region

#Region " Private Methods "
        Private Sub initialiseXmlDocument()
            If System.IO.File.Exists(m_sFilename) Then
                Try
                    Dim xmltrSettings As New XmlTextReader(m_sFilename)
                    m_xmlDocument = New XmlDocument
                    m_xmlDocument.Load(xmltrSettings)
                    xmltrSettings.Close()
                    xmltrSettings = Nothing
                Catch exError As Exception
                    Debug.WriteLine(exError.ToString)
                End Try
            Else
                m_xmlDocument = New XmlDocument
            End If
        End Sub
#End Region

#Region " Public Methods "
        Public Sub loadSettings()
            Dim sSectionName As String
            Dim xnlSections As XmlNodeList
            Dim xnSection As XmlNode
            Dim oSection As Section

            xnlSections = m_xmlDocument.SelectNodes("//Section")
            If Not xnlSections Is Nothing Then
                For Each xnSection In xnlSections
                    sSectionName = xnSection.Attributes("Name").Value
                    Debug.WriteLine("Found Section: " & sSectionName)
                    oSection = New Section(m_xmlDocument)
                    oSection.loadSection(sSectionName)
                    m_htSettings.Add(sSectionName, oSection)
                    oSection = Nothing
                Next
            End If

            xnSection = Nothing
            xnlSections = Nothing
        End Sub
        Public Sub saveSettings()
            Dim deItems As IDictionaryEnumerator

            ' Create XML Document
            If m_xmlDocument.DocumentElement Is Nothing Then
                Try
                    m_xmlDocument.LoadXml("<?xml version=""1.0"" encoding=""UTF-8""?>" & ControlChars.CrLf & _
                        "<ApplicationSettings>" & ControlChars.CrLf & "</ApplicationSettings>")
                Catch exError As Exception
                    Debug.WriteLine("saveSettings :: " & exError.ToString)
                End Try
            End If

            ' Save Sections
            deItems = Me.Items.GetEnumerator
            While deItems.MoveNext
                CType(deItems.Value, Section).saveSection()
            End While

            ' Write new XML Document
            Try
                m_xmlDocument.Save(Me.Filename)
            Catch exError As Exception
                Debug.WriteLine("saveSettings :: " & exError.ToString)
            End Try
        End Sub

        Private Function getValue(ByVal sSectionName As String, ByVal sKeyName As String, Optional ByVal bAddNonexisting As Boolean = False, Optional ByVal sSetValue As String = "") As String
            Dim oSection As Section
            Dim oValue As Value
            Try
                oSection = CType(Me.Items(sSectionName), Section)
                oValue = CType(oSection.Items(sKeyName), Value)
                Return oValue.Value
            Catch
                ' Not Found!
                If bAddNonexisting Then
                    If Not m_htSettings.ContainsKey(sSectionName) Then
                        ' Create Section
                        oSection = New Section(m_xmlDocument)
                        oSection.Name = sSectionName
                        m_htSettings.Add(sSectionName, oSection)
                    Else
                        oSection = CType(Me.Items(sSectionName), Section)
                    End If

                    If Not oSection.ItemExists(sKeyName) Then
                        ' Create Value + Force Update
                        oValue = New Value(sKeyName, sSetValue)
                        oValue.ForceUpdate()
                        oSection.Items.Add(sKeyName, oValue)
                    Else
                        CType(oSection.Items(sKeyName), Value).ForceUpdate()
                    End If

                    oValue = Nothing
                    oSection = Nothing
                End If

                Return String.Empty
            End Try
        End Function
        Public Function getValueInteger(ByVal sSectionName As String, ByVal sKeyName As String, Optional ByVal iDefaultValue As Integer = 0, Optional ByVal bAddNonexisting As Boolean = False) As Integer
            Dim sValue As String = getValue(sSectionName, sKeyName, bAddNonexisting, iDefaultValue.ToString)
            If sValue Is String.Empty Then
                Return iDefaultValue
            Else
                Return CType(sValue, Integer)
            End If
        End Function
        Public Function getValueString(ByVal sSectionName As String, ByVal sKeyName As String, Optional ByVal sDefaultValue As String = "", Optional ByVal bAddNonexisting As Boolean = False) As String
            Dim sValue As String = getValue(sSectionName, sKeyName, bAddNonexisting, sDefaultValue)
            If sValue Is String.Empty Then
                Return sDefaultValue
            Else
                Return sValue
            End If
        End Function
        Public Function getValueBoolean(ByVal sSectionName As String, ByVal sKeyName As String, Optional ByVal bDefaultValue As Boolean = False, Optional ByVal bAddNonexisting As Boolean = False) As Boolean
            Dim sValue As String = getValue(sSectionName, sKeyName, bAddNonexisting, bDefaultValue.ToString)
            If sValue Is String.Empty Then
                Return bDefaultValue
            Else
                Return CType(sValue, Boolean)
            End If
        End Function

        Public Sub setValue(ByVal sSectionName As String, ByVal sKeyName As String, ByVal sValue As String)
            Dim oSection As Section
            Dim oValue As Value

            If m_htSettings.ContainsKey(sSectionName) Then
                oSection = CType(m_htSettings(sSectionName), Section)
                If oSection.ItemExists(sKeyName) Then
                    oValue = CType(oSection.Items(sKeyName), Value)
                    oValue.Value = sValue
                End If
            End If
        End Sub
        Public Sub setValue(ByVal sSectionName As String, ByVal sKeyName As String, ByVal iValue As Integer)
            setValue(sSectionName, sKeyName, CType(iValue, String))
        End Sub
        Public Sub setValue(ByVal sSectionName As String, ByVal sKeyName As String, ByVal bValue As Boolean)
            setValue(sSectionName, sKeyName, CType(CType(bValue, Integer), String))
        End Sub
#End Region

#Region " Public Properties "
        Public ReadOnly Property Items() As Hashtable
            Get
                Return m_htSettings
            End Get
        End Property
        Public ReadOnly Property ItemExists(ByVal sSectionName As String) As Boolean
            Get
                Return m_htSettings.ContainsKey(sSectionName)
            End Get
        End Property
        Public Property Filename() As String
            Get
                Return m_sFilename
            End Get
            Set(ByVal sFilename As String)
                m_sFilename = sFilename
            End Set
        End Property
#End Region

    End Class
#End Region

#Region " Class: Section "
    ' Section
    Public Class Section
        Private m_htValues As Hashtable
        Private m_xmlDocument As XmlDocument
        Private m_sName As String

#Region " Constructor "
        Sub New(ByRef xmlSettingsDocument As XmlDocument)
            m_xmlDocument = xmlSettingsDocument
            m_htValues = New Hashtable
        End Sub
#End Region

#Region " Public Properties  "
        Public ReadOnly Property Items() As Hashtable
            Get
                Return m_htValues
            End Get
        End Property
        Public ReadOnly Property ItemExists(ByVal sKeyName As String) As Boolean
            Get
                Return m_htValues.ContainsKey(sKeyName)
            End Get
        End Property
        Public Property Name() As String
            Get
                Return m_sName
            End Get
            Set(ByVal sName As String)
                m_sName = sName
            End Set
        End Property
#End Region

#Region " Public Methods"

        Public Sub loadSection(ByVal sSectionName As String)
            Dim xnSection, xnKey As XmlNode
            Dim xnlKeys As XmlNodeList
            Dim oValue As Value

            Me.Name = sSectionName
            xnSection = m_xmlDocument.SelectSingleNode("//Section[@Name='" & sSectionName & "']")
            If Not xnSection Is Nothing Then
                xnlKeys = xnSection.SelectNodes("descendant::Key")
                If Not xnlKeys Is Nothing Then
                    For Each xnKey In xnlKeys
                        ' Add Value
                        oValue = New Value(xnKey.Attributes("Name").Value, xnKey.Attributes("Value").Value)
                        m_htValues.Add(oValue.Name, oValue)
                        Debug.WriteLine("Added Value: " & oValue.Name & " = " & oValue.Value)
                    Next
                End If
            End If
            xnlKeys = Nothing
            xnSection = Nothing
        End Sub

        Public Sub saveSection()
            Dim xnRoot, xnSection As XmlNode
            Dim xnAttr As XmlAttribute
            Dim deItems As IDictionaryEnumerator = Me.Items.GetEnumerator
            Dim oValue As Value

            ' Check if section exists
            Debug.WriteLine("SaveSection (" & Me.Name & ")")
            xnSection = m_xmlDocument.SelectSingleNode("//Section[@Name='" & Me.Name & "']")
            If xnSection Is Nothing Then
                Try
                    ' Create Section
                    xnSection = m_xmlDocument.CreateNode(XmlNodeType.Element, "Section", "")

                    ' Add "Name" Attribute
                    xnAttr = m_xmlDocument.CreateAttribute("Name")
                    xnAttr.Value = Me.Name
                    xnSection.Attributes.SetNamedItem(xnAttr)

                    ' Add Section to Root Node
                    xnRoot = m_xmlDocument.DocumentElement
                    xnRoot.AppendChild(xnSection)
                    xnRoot = Nothing

                    Debug.WriteLine("SaveSection :: Section """ & Me.Name & """ created.")
                Catch exError As Exception
                    Debug.WriteLine("saveSection :: " & exError.ToString)
                End Try
            End If

            ' Save Keys/Values
            While deItems.MoveNext
                oValue = CType(deItems.Value, Value)
                If oValue.Changed Then
                    saveKey(xnSection, CType(deItems.Value, Value))
                End If
            End While

            xnSection = Nothing
        End Sub

        Public Sub saveKey(ByRef xnSection As XmlNode, ByVal oKey As Value)
            Dim xnKey As XmlNode
            Dim xnAttr As XmlAttribute

            ' Check if Key exists
            Debug.WriteLine("saveKey (" & oKey.Name & ")")
            xnKey = xnSection.SelectSingleNode("descendant::Key[@Name='" & oKey.Name & "']")
            If xnKey Is Nothing Then
                Try
                    'create new node
                    xnKey = m_xmlDocument.CreateNode(XmlNodeType.Element, "Key", "")

                    ' Add "Name" Attribute
                    xnAttr = m_xmlDocument.CreateAttribute("Name")
                    xnAttr.Value = oKey.Name
                    xnKey.Attributes.SetNamedItem(xnAttr)

                    ' Add "Value" Attribute
                    xnAttr = m_xmlDocument.CreateAttribute("Value")
                    xnAttr.Value = oKey.Value
                    xnKey.Attributes.SetNamedItem(xnAttr)

                    ' Add the new node to its Section
                    xnSection.AppendChild(xnKey)

                    Debug.WriteLine("saveKey :: Key """ & oKey.Name & """ created.")
                Catch exError As Exception
                    Debug.WriteLine("saveKey :: " & exError.ToString)
                End Try
            Else
                xnKey.Attributes("Value").Value = oKey.Value
            End If
        End Sub

#End Region

    End Class
#End Region

#Region " Class: Value "
    Public Class Value
        Private m_sName As String
        Private m_sValue As String
        Private m_bChanged As Boolean

#Region " Constructor "
        Public Sub New(ByVal sName As String, ByVal sValue As String)
            m_sName = sName
            m_sValue = sValue
        End Sub
#End Region

#Region " Public Properties "

        Public Property Name() As String
            Get
                Return m_sName
            End Get
            Set(ByVal sName As String)
                m_sName = sName
            End Set
        End Property

        Public Property Value() As String
            Get
                Return m_sValue
            End Get
            Set(ByVal sValue As String)
                If sValue <> m_sValue Then
                    m_bChanged = True
                End If

                m_sValue = sValue
            End Set
        End Property

        Public ReadOnly Property Changed() As Boolean
            Get
                Return m_bChanged
            End Get
        End Property

        Public Sub ForceUpdate()
            m_bChanged = True
        End Sub

#End Region

    End Class
#End Region

End Namespace