﻿Option Explicit On
Option Strict On

Imports System.Runtime.InteropServices
Imports System.Configuration
Imports System.Threading

Public Class frmMain

    Private Game As Sezz.AION.Game
    Private WithEvents Armory As Sezz.AION.Armory
    Private bEntityInRange As Boolean = False ' TODO: Remove AionArmory

    ' Statistics
    Private iItemsGathered As Integer = 0
    Private iStartExperience As Integer = 0

    ' Multi-Threading/Timers
    Private bRefreshGameStatusInProgress As Boolean
    Private bGathererEnabled As Boolean = False
    Private oGathererThread As Thread
    Private WithEvents tmrGameInformation As System.Windows.Forms.Timer

#Region " Loading/Closing Events "
    Private Sub frmMain_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Game = New Sezz.AION.Game() ' AION Game Object
        Game.Initialize()
        Armory = New Sezz.AION.Armory() 'AION Armory Web Client
        Armory.LoadCache()
        ReadConfiguration() ' Load Client Configuration

        Me.Icon = GetIconResource("Coffee.ico")

        ' TextBox Coloring Workaround
        txtAionProcessID.BackColor = Color.FromName("Control")
        txtGameDLLBaseAddress.BackColor = Color.FromName("Control")
        txtCurrentlyGathering.BackColor = Color.FromName("Control")
        txtNearestResourceDistace.BackColor = Color.FromName("Control")
        txtPlayerStatus.BackColor = Color.FromName("Control")

        ' Item Colors
        ' Taken from AionArmory
        ItemRarityColors(0) = SystemColors.GrayText        ' Color.FromArgb(213, 213, 213) ' Junk
        ItemRarityColors(1) = SystemColors.WindowText      ' Color.FromArgb(255, 255, 255) ' Common
        ItemRarityColors(2) = Color.FromArgb(74, 185, 59)  ' Superior
        ItemRarityColors(3) = Color.FromArgb(22, 153, 196) ' Heroic
        ItemRarityColors(4) = Color.FromArgb(255, 153, 0)  ' Fabled
        ItemRarityColors(5) = Color.FromArgb(146, 56, 208) ' Eternal
        ItemRarityColors(6) = Color.FromArgb(241, 127, 6)  ' ?
        ItemRarityColors(7) = Color.FromArgb(241, 127, 6)  ' ?

        ' About Window Text
        lblAppName.Text = Application.ProductName
        lblAppVersion.Text = "Version: " & System.Reflection.Assembly.GetExecutingAssembly().GetName().Version.Major & "." & _
            System.Reflection.Assembly.GetExecutingAssembly().GetName().Version.Minor & " (Build " & _
            System.Reflection.Assembly.GetExecutingAssembly().GetName().Version.Build & ")"
        lblAppVersionCompiled.Text = "Compiled: " & AssemblyCreationDate.ToString

        ' Initialize & Go ;)
        CheckForIllegalCrossThreadCalls = False
        ResetStatistics()
        RefreshGameStatus(Nothing, Nothing)
        tmrGameInformation = New System.Windows.Forms.Timer()
        AddHandler tmrGameInformation.Tick, AddressOf RefreshGameStatus
        With tmrGameInformation
            .Interval = 500
            .Enabled = True
        End With
    End Sub

    Private Sub frmMain_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        bGathererEnabled = False

        ' Update Armory Cache + Save Configuration
        Armory.UpdateCache()
        SaveConfiguration()
    End Sub
#End Region

#Region " Botting Thread + Control "
    Private Sub cmdStartStop_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdStartStop.Click
        If oGathererThread Is Nothing OrElse oGathererThread.ThreadState = ThreadState.Stopped Then
            ' Bot is stopped... start it!
            Debug.WriteLine("START")
            oGathererThread = New Thread(AddressOf Gatherer)
            bGathererEnabled = True
            cmdStartStop.Text = "STOP"
            oGathererThread.Start()
        ElseIf oGathererThread.IsAlive Then
            ' Bot is running... stop it!
            bGathererEnabled = False
            cmdStartStop.Text = "START"
            Debug.WriteLine("STOP")
        End If
    End Sub

    Private Sub Gatherer()
        Dim bRecentlyGathered As Boolean = False
        Dim iExperienceBeforeGathering As Integer = Integer.MaxValue

        While bGathererEnabled
            If Game.Running AndAlso Game.Player.IsLoggedIn Then
                If Not Game.Player.IsAlive Then
                    bGathererEnabled = False
                Else
                    If iExperienceBeforeGathering = Integer.MaxValue Then
                        iExperienceBeforeGathering = Game.Player.CompleteExperience
                    End If

                    If bEntityInRange AndAlso Not Game.Player.IsGathering Then
                        ' Resource Node in Range, Player idle -> Gather
                        If bRecentlyGathered Then
                            ' Cooldown
                            'Debug.WriteLine("1?")
                            SleepWhileEnabled(500)
                            If iExperienceBeforeGathering <> Game.Player.CompleteExperience Then
                                ' Gathering was successful!
                                'Debug.WriteLine("succ 1")
                                iItemsGathered += 1
                            Else
                                'Debug.WriteLine("epic fail!")
                            End If
                            SleepWhileEnabled(3000)
                            bRecentlyGathered = False
                        End If
                        If bEntityInRange AndAlso Not Game.Player.IsGathering Then
                            ' Gather
                            iExperienceBeforeGathering = Game.Player.CompleteExperience
                            Game.SendKey(ApplicationSettings.GatheringKey(0), ApplicationSettings.BackgroundMode)
                            bRecentlyGathered = True
                        End If
                    ElseIf bEntityInRange AndAlso Game.Player.IsGathering Then
                        ' Resource Node in Range, Player is gathering -> Fine ;)
                        bRecentlyGathered = True
                    Else
                        ' Idle
                        If bRecentlyGathered Then
                            'Debug.WriteLine("2?")
                            SleepWhileEnabled(500)
                            If iExperienceBeforeGathering <> Game.Player.CompleteExperience Then
                                ' Gathering was successful!
                                'Debug.WriteLine("succ 2")
                                iItemsGathered += 1
                            Else
                                'Debug.WriteLine("epic fail!")
                            End If
                            bRecentlyGathered = False
                        End If
                    End If
                End If
            End If

            SleepWhileEnabled(ApplicationSettings.UpdateInterval)
        End While
    End Sub

    Public Sub SleepWhileEnabled(ByVal lMilliseconds As Long)
        Dim tEndOfSleep As Date = Now().AddMilliseconds(lMilliseconds)

        While Now < tEndOfSleep AndAlso bGathererEnabled
            System.Threading.Thread.Sleep(RandomNumber(50, 150))
        End While
    End Sub
#End Region

#Region " Configuration "
    Private Sub pgApplicationSettings_PropertyValueChanged(ByVal s As Object, ByVal e As System.Windows.Forms.PropertyValueChangedEventArgs) Handles pgApplicationSettings.PropertyValueChanged
        If e.ChangedItem.Label = "GatheringResourceName" Then
            txtWantedResource.Text = ApplicationSettings.GatheringResourceName
        ElseIf e.ChangedItem.Label = "AlwaysOnTop" Then
            Me.TopMost = ApplicationSettings.AlwaysOnTop
        End If
    End Sub

    Private Sub ReadConfiguration()
        ApplicationSettingsXML = New Sezz.XML.Settings() ' Read Configuration from XML File
        ApplicationSettings = New Sezz.ApplicationSettings(ApplicationSettingsXML) ' Stupid Class for the Property Grid
        pgApplicationSettings.SelectedObject = ApplicationSettings
        txtWantedResource.Text = ApplicationSettings.GatheringResourceName
        Me.TopMost = ApplicationSettings.AlwaysOnTop
    End Sub

    Private Sub SaveConfiguration()
        txtWantedResource.Text = ApplicationSettings.GatheringResourceName
        ApplicationSettingsXML.setValue("Gathering", "ResourceName", ApplicationSettings.GatheringResourceName)
        ApplicationSettingsXML.setValue("Gathering", "Key", ApplicationSettings.GatheringKey)
        ApplicationSettingsXML.setValue("Gathering", "Range", ApplicationSettings.GatheringRange)
        ApplicationSettingsXML.setValue("Main", "AlwaysOnTop", ApplicationSettings.AlwaysOnTop)
        ApplicationSettingsXML.setValue("Main", "BackgroundMode", ApplicationSettings.BackgroundMode)
        ApplicationSettingsXML.setValue("Main", "UpdateInterval", ApplicationSettings.UpdateInterval)
        ApplicationSettingsXML.saveSettings()
    End Sub
#End Region

#Region " User Interface "
    Private Sub UpdateTextBox(ByRef oOldTextBox As TextBox, ByVal sNewText As String, ByVal oNewForeColor As Color, ByVal oNewBackColor As Color)
        If oOldTextBox.Text <> sNewText Then oOldTextBox.Text = sNewText
        If oOldTextBox.ForeColor <> oNewForeColor Then oOldTextBox.ForeColor = oNewForeColor
        If oOldTextBox.BackColor <> oNewBackColor Then oOldTextBox.BackColor = oNewBackColor
    End Sub

    Private Sub UpdateTextBox(ByRef oOldTextBox As TextBox, ByVal sNewText As String, ByVal oNewForeColor As Color)
        If oOldTextBox.Text <> sNewText Then oOldTextBox.Text = sNewText
        If oOldTextBox.ForeColor <> oNewForeColor Then oOldTextBox.ForeColor = oNewForeColor
    End Sub

    Private Sub UpdateTextBox(ByRef oOldTextBox As TextBox, ByVal sNewText As String)
        If oOldTextBox.Text <> sNewText Then oOldTextBox.Text = sNewText
    End Sub
#End Region

#Region " Game Information/Statistics Timer "
    Private Sub RefreshGameStatus(ByVal sender As System.Object, ByVal e As System.EventArgs)
        If bRefreshGameStatusInProgress Then
            ' Should never (?) occur.
            Debug.WriteLine("RefreshGameStatus already in progress - skipped!")
        Else
            bRefreshGameStatusInProgress = True

            Dim oItemGathering As Sezz.AION.Armory.Data
            ' Display AION Process Information
            Game.Refresh()
            If Game.Running() Then
                UpdateTextBox(txtAionProcessID, Game.ProcessID.ToString, Color.Green, SystemColors.Control)
                UpdateTextBox(txtGameDLLBaseAddress, "0x" & Game.GameDllBaseAddress.ToString("X"), Color.Green)
            Else
                UpdateTextBox(txtAionProcessID, "UNKNOWN", Color.White, Color.Red)
                UpdateTextBox(txtGameDLLBaseAddress, "", Color.Green)
            End If

            ' Display Player Status
            If Game.Player.Update() Then
                UpdateTextBox(txtPlayerName, Game.Player.FullName)

                ' Update nearest Resource Node Distance
                UpdateResourceNodes()

                ' Gathering Info
                If Game.Player.IsGathering Then
                    UpdateTextBox(txtPlayerStatus, "Gathering", Color.Green, SystemColors.Control)

                    If Game.Player.GatheringItemId < 1000000 Then
                        ' No Idea
                    Else
                        oItemGathering = Armory.GetData(Game.Player.GatheringItemId, Sezz.AION.Armory.RequestType.Item, True)
                        If oItemGathering.Name.Length = 0 Then
                            ' Downloading Data in Background - Display Item ID
                            oItemGathering.Rarity = 0
                            oItemGathering.Name = oItemGathering.ID.ToString
                        End If

                        If oItemGathering.Name = "0" Then Debug.WriteLine("****************************")
                        If oItemGathering.Name = "0" Then Debug.WriteLine(Game.Player.GatheringItemId)
                        If oItemGathering.Name = "0" Then Debug.WriteLine("****************************")

                        UpdateTextBox(txtCurrentlyGathering, oItemGathering.Name, ItemRarityColors(oItemGathering.Rarity))
                    End If
                Else
                    UpdateTextBox(txtCurrentlyGathering, "")

                    If Game.Player.IsAlive Then
                        UpdateTextBox(txtPlayerStatus, "Idle", SystemColors.WindowText, SystemColors.Control)
                    Else
                        UpdateTextBox(txtPlayerStatus, "DEAD", Color.White, Color.Red)
                    End If
                End If
            Else
                UpdateTextBox(txtPlayerName, "")
                UpdateTextBox(txtPlayerStatus, "")
                UpdateTextBox(txtCurrentlyGathering, "")
                UpdateTextBox(txtNearestResourceDistace, "")
            End If

            'UpdateStatistics()
            bRefreshGameStatusInProgress = False
        End If
    End Sub

    Private Sub UpdateResourceNodes()
        Dim dDistance As Double = Double.MaxValue
        Dim dNearestDistance As Double = Double.MaxValue

        bEntityInRange = False

        Game.EntityList.Update()
        For Each Entity As Sezz.AION.Entity In Game.EntityList
            If Entity.Name = ApplicationSettings.GatheringResourceName Then
                dDistance = Game.Player.DistanceTo(Entity)
                If dDistance > 1 AndAlso dDistance < dNearestDistance Then dNearestDistance = dDistance
                If dNearestDistance <= ApplicationSettings.GatheringRange Then
                    bEntityInRange = True
                    Exit For
                End If
            End If
        Next

        If dDistance <> Double.MaxValue Then
            If bEntityInRange Then
                UpdateTextBox(txtNearestResourceDistace, Math.Round(dNearestDistance, 2).ToString("#.00"), Color.Green)
            Else
                UpdateTextBox(txtNearestResourceDistace, Math.Round(dNearestDistance, 2).ToString("#.00"), Color.DarkRed)
            End If
        Else
            UpdateTextBox(txtNearestResourceDistace, "NA", Color.Red)
        End If
    End Sub
#End Region

#Region " AionArmory Multi-Threading "
    Private Sub DownloadFinished(ByVal oData As Sezz.AION.Armory.Data) Handles Armory.DownloadFinished
        If oData.ID > 0 AndAlso Game.Player.GatheringItemId() = oData.ID AndAlso oData.Name.Length > 0 Then
            UpdateTextBox(txtCurrentlyGathering, oData.Name, ItemRarityColors(oData.Rarity))
        End If
    End Sub
#End Region

#Region " Statistics "
    Public Sub UpdateStatistics()
        If bGathererEnabled Then
            If Game.Running And Game.Player.IsLoggedIn Then
                If iStartExperience = Integer.MaxValue Then
                    iStartExperience = Game.Player.CompleteExperience
                End If

                UpdateTextBox(txtStatisticsXPGained, (Game.Player.CompleteExperience - iStartExperience).ToString("#,###,##0"))
            End If

            UpdateTextBox(txtStatisticsItemsGathered, iItemsGathered.ToString("#,###,##0"))
        End If
    End Sub

    Private Sub ResetStatistics()
        iItemsGathered = 0
        If Game.Running And Game.Player.IsLoggedIn Then
            iStartExperience = Game.Player.CompleteExperience
        Else
            iStartExperience = Integer.MaxValue
        End If

        UpdateTextBox(txtStatisticsItemsGathered, "0")
        UpdateTextBox(txtStatisticsXPGained, "0")
    End Sub
    Private Sub cmdStatisticsReset_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdStatisticsReset.Click
        ResetStatistics()
    End Sub
#End Region

End Class
